import numpy as np
import pytest
import kdtree
from sklearn.neighbors import KDTree


@pytest.mark.parametrize("n_samples", [10, 100, 1000, 10000])
@pytest.mark.parametrize("n_features", [10, 100])
@pytest.mark.parametrize("leaf_size", [256, 1024])
def test_creation_deletion(n_samples, n_features, leaf_size):
    rng = np.random.RandomState(1)
    X = rng.rand(n_samples, n_features)

    tree = kdtree.KDTree(X, leaf_size=256)
    del tree

@pytest.mark.parametrize("n_samples", [10, 100, 1000])
@pytest.mark.parametrize("n_features", [10, 100])
@pytest.mark.parametrize("k", [1, 2, 5, 10])
@pytest.mark.parametrize("leaf_size", [256, 1024])
def test_against_sklearn(n_samples, n_features, k, leaf_size, n_query=10):
    rng = np.random.RandomState(1)
    X = rng.rand(n_samples, n_features)
    query_points = rng.rand(n_query, n_features)

    tree = kdtree.KDTree(X, leaf_size=256)
    skl_tree = KDTree(X, leaf_size=256)

    knn_indices = np.zeros((n_query, k), dtype=np.int32)
    knn_distances = np.zeros((n_query, k), dtype=np.float64)
    tree.query(query_points, knn_indices, knn_distances)
    skl_knn_distances, skl_knn_indices = skl_tree.query(
        query_points,
        k=k,
        return_distance=True
    )

    # Adapting types
    skl_knn_indices = skl_knn_indices.astype(np.int32)

    np.testing.assert_equal(knn_indices, skl_knn_indices)
    # np.testing.assert_almost_equal(knn_distances, skl_knn_distances)
